<?php

namespace Itguild\Tables;

use Exception;
use Itguild\Tables\ActionColumn\ActionColumn;
use Itguild\Tables\ActionColumn\DeleteActionColumn;
use Itguild\Tables\ActionColumn\EditActionColumn;
use Itguild\Tables\ActionColumn\ViewActionColumn;
use Itguild\Tables\traits\CreateParams;
use JetBrains\PhpStorm\NoReturn;

class ListJsonTable
{
    use CreateParams;

    private string $html = '';
    private string $json;

    private int $count = 0;
    private \Closure|false $beforePrintCell;
    private \Closure|false $beforePrintHook;
    private \Closure|false $afterPrintHook;
    private string $baseUrl;
    private array $data;

    private bool $pagination = true;
    private bool $showActionColumn = true;
    private bool $filter = true;

    private array $actionsArray = [];
    private array $customActionsArray = [];
    private array $customColumnsArray = [];

    #[NoReturn] public function __construct(string $json)
    {
        $this->beforePrintCell = false;
        $this->json = $json;
        $this->data = json_decode($this->json, true);
        $this->baseUrl = $this->data['meta']['baseUrl'] ?? '';
        $this->pagination = $this->data['meta']['pagination'] ?? true;
        $this->showActionColumn = $this->data['meta']['showActionColumn'] ?? true;
        $this->filter = $this->data['meta']['filter'] ?? true;
        $this->beforePrintHook = function () {
        };
        $this->afterPrintHook = function () {
        };
    }

    public function beginTable(): void
    {
        $paramsStr = $this->createParams($this->data['meta']['params']);
        $hookBefore = $this->beforePrintHook;
        $this->html .= $hookBefore($this->data['meta']);
        $this->html .= "<table $paramsStr>";
    }

    public function beforePrint(\Closure $closure): void
    {
        $this->beforePrintHook = $closure;
    }

    public function afterPrint(\Closure $closure): void
    {
        $this->afterPrintHook = $closure;
    }

    public function createThead(): void
    {
        if (!isset($this->data['meta']['columns'])) {
            return;
        }

        $this->html .= "<thead><tr>";
        $this->html .= "<th>" . "ID" . "</th>";
        foreach ($this->data['meta']['columns'] as $key => $column) {
            if ($this->is_fillable($key)) {
                $this->html .= "<th>" . $column . "</th>";
            }
        }
        $this->getCustomHeadColumn();
        if ($this->showActionColumn) {
            $this->html .= "<th>Действия</th></th></tr></thead>";
        }
    }

    public function createTbody(): void
    {
        if ($this->data['data']) {

//            if($this->filter)
//            {
//                foreach ($this->data['meta']['columns'] as $col) {
//                    $this->html .= "<tr><td>";
//                    foreach ($this->data['meta']['filters'] as $filter) {
//                        if ($this->issetFilter($filter))
//                        {
//                            $filters = new Filter($filter, )
//                        }
//                    }
//
//
//                }
//            }
            $this->count = $this->data["meta"]["perPage"] * ($this->data['meta']["currentPage"] - 1);
            foreach ($this->data['data'] as $row) {
                $this->html .= "<tr>";
                $this->count += 1;
                $id = $row["id"] ?? $this->count;
                $this->html .= '<td><a href=' . $this->baseUrl . "/" . $id . '>' . $id . '</a></td>';
                foreach ($row as $key => $ceil) {
                    if ($this->issetColumn($key) and $this->is_fillable($key)) {
                        if ($this->beforePrintCell) {
                            $hook = $this->beforePrintCell;
                            $ceil = $hook($key, $ceil);
                        }

                        $this->html .= "<td>" . $ceil . "</td>";
                    }
                }

                $this->getCustomColumns($row["id"] ?? null);

                if ($this->showActionColumn) {
                    if (isset($row["id"])) {
                        $actions = $this->getActions($row["id"]);

                        $this->html .= "<td>$actions</td></tr>";
                    } else {
                        $this->html .= "<td></td></tr>";
                    }
                }
            }
        }
    }

    public function addAction(string $actionColumn): void
    {
        $this->customActionsArray[] = $actionColumn;
    }

    public function addColumn(string $label, string $key, \Closure $closure): void
    {
        $this->customColumnsArray[] = ['label' => $label, "key" => $key, "handler" => $closure];
    }


    private function setActions(): void
    {
        if (isset($this->data['meta']['actions'])) {
            foreach ($this->data['meta']['actions'] as $action) {
                switch ($action) {
                    case 'view':
                        $this->actionsArray[] = ViewActionColumn::class;
                        break;
                    case 'delete':
                        $this->actionsArray[] = DeleteActionColumn::class;
                        break;
                    case 'edit':
                        $this->actionsArray[] = EditActionColumn::class;
                        break;
                }
            }
        }
        $this->actionsArray = array_merge($this->actionsArray, $this->customActionsArray);
    }

    public function getCustomColumns($id = null): void
    {
        foreach ($this->customColumnsArray as $item) {
            $this->html .= "<td>".$item['handler']($id)."</td>";
        }
    }

    public function getCustomHeadColumn(): void
    {
        foreach ($this->customColumnsArray as $item) {
            $this->html .= "<th>".$item['label']."</th>";
        }
    }

    private function issetColumn($column): bool
    {
        if (isset($this->data['meta']['columns'])) {
            foreach ($this->data['meta']['columns'] as $key => $currentColumn) {
                if ($key === $column) {
                    return true;
                }
            }
        }

        return false;
    }

    private function issetFilter($filter): bool
    {
        if (isset($this->data['meta']['filter'])) {
            foreach ($this->data['meta']['filter'] as $key => $currentFilter) {
                if ($key === $filter) {
                    return true;
                }
            }
        }
        return false;
    }

    private function is_fillable($column): bool
    {
        if (isset($this->data['meta']['fillable'])) {
            if (in_array($column, $this->data['meta']['fillable'])) {
                return true;
            }
        } else {
            return true;
        }

        return false;
    }

//    private function getColumns(int $id): string
//    {
//        $columns = [];
//        foreach ($this->actionsArray as $item) {
//            $objItem = new $item($this->baseUrl, $id);
//            $columns .= $objItem->fetch();
//        }
//
//        return $columns;
//    }

    private function getActions(int $id): string
    {
        $actions = "";
        foreach ($this->actionsArray as $item) {
            $objItem = new $item($this->baseUrl, $id);
            $actions .= $objItem->fetch();
        }

        return $actions;
    }

    /**
     * @throws Exception
     */
    public function create(): void
    {
        $this->setActions();
        $this->beginTable();
        $this->createThead();
        $this->createTbody();
        $this->endTable();
    }

    public function tableAction($json): void
    {
        $tableJson = json_decode($json, true);

        foreach ($tableJson as $key => $value) {
            echo "<tr>";
            echo "<td>" . $key . "</td>";
            echo "<td>" . $value . "</td>";
            echo "</tr>";
        }
    }

    /**
     * @throws Exception
     */
    public function endTable(): void
    {
        $this->html .= "</table>";
        $hookAfter = $this->afterPrintHook;
        $this->html .= $hookAfter($this->data['meta']);

        if ($this->pagination) {
            $options = [
                'countItem' => $this->data['meta']['total'],
                'perPage' => $this->data['meta']['perPage'],
                'currentPage' => $this->data['meta']['currentPage'],
                'baseUrl' => $this->baseUrl,
                'prefix' => $this->data['meta']['paginationPrefix'],
            ];
            $pagination = new Pagination($options);
            $pagination->create();
            $this->html .= $pagination->fetch();
        }
    }

    public function render(): void
    {
        echo $this->html;
    }

    public function setBeforePrintCell(\Closure $closure): void
    {
        $this->beforePrintCell = $closure;
    }
}